@@ -19,6 +19,7 @@ KEY__deletion_key = "deletion_key"
 KEY__render_as = "render_as"
 KEY__default_tenant_password = "default_tenant_password"
 KEY__url = "url"
+KEY__artifact_base_uri = "artifact_base_uri"
 KEY__superjaaql_password = "superjaaql_password"
 KEY__public_username = "public_username"
 KEY__records_total = "records_total"
@@ -60,6 +61,7 @@ KEY__dataset_name = "name"
 KEY__dataset_description = "dataset_description"
 KEY__size = "size"
 KEY__database = "database"
+KEY__read_only = "read_only"
 KEY__node_name = "name"
 KEY__authorization = "authorization"
 KEY__argument = "argument"
@@ -69,6 +71,16 @@ KEY__connection = "connection"
 KEY__node = "node"
 KEY__application_url = "url"
 KEY__key = "key"
+KEY__override_send_name = "override_send_name"
+KEY__ip = "ip"
+KEY__status = "status"
+KEY__endpoint = "endpoint"
+KEY__input = "input"
+KEY__duration_ms = "duration_ms"
+KEY__occurred = "occurred"
+KEY__jaaql_enc_symmetric_key = "jaaql_enc_symmetric_key"
+KEY__enc_symmetric_key = "enc_symmetric_key"
+KEY__default_enc_symmetric_key = "default_enc_symmetric_key"
 KEY__role = "role"
 KEY__mfa_key = "mfa_key"
 KEY__mfa_enabled = "mfa_enabled"
@@ -139,6 +151,14 @@ KEY__email_account_send_name = "send_name"
 KEY__encrypted_address = "encrypted_address"
 KEY__password_id = "password_id"
 KEY__created = "created"
+KEY__source_file = "source_file"
+KEY__position = "position"
+KEY__file = "file"
+KEY__error = "error"
+KEY__error_row_number = "row_number"
+KEY__error_index = "index"
+KEY__error_query = "query"
+KEY__error_set = "set"
 
 ATTR__deleted = "deleted"
 
@@ -174,6 +194,7 @@ DIR__www = "www"
 DIR__render_template = "rendered_documents"
 
 FILE__config = "config.ini"
+FILE__canned_queries = "canned_queries.jsql"
 
 ENCODING__utf = "UTF-8"
 ENCODING__ascii = "ascii"
@@ -192,9 +213,11 @@ ENVIRON__jaaql_profiling = "JAAQL_PROFILING"
 ENVIRON__email_credentials = "JAAQL_EMAIL_CREDENTIALS"
 ENVIRON__install_path = "INSTALL_PATH"
 ENVIRON__sentinel_url = "SENTINEL_URL"
+ENVIRON__canned_queries = "CANNED_QUERIES"
 
 EMAIL_PARAM__signup_key = "JAAQL_INVITE_KEY"
 EMAIL_PARAM__invite_code = "JAAQL_INVITE_SHORT_CODE"
+EMAIL_PARAM__app_url = "APP_URL"
 
 EMAIL_PARAM__reset_key = "JAAQL_RESET_KEY"
 EMAIL_PARAM__reset_code = "JAAQL_RESET_SHORT_CODE"
@@ -206,6 +229,7 @@ JWT_PURPOSE__oauth = "oauth"
 JWT_PURPOSE__pre_auth = "pre_auth"
 JWT_PURPOSE__connection = "connection"
 
+ERR__invalid_token = "Invalid token!"
 ERR__not_yet_installed = "JAAQL has not yet been installed!"
 ERR__user_public = "Cannot perform this action on a public user!"
 ERR__too_many_signup_attempts = "Too many signup attempts"
@@ -251,4 +275,5 @@ ROLE__postgres = "postgres"
 
 PROTOCOL__postgres = "postgresql://"
 
-VERSION = "3.2.0"
+VERSION = "4.1.2"
+
